var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var move_exports = {};
__export(move_exports, {
  SortAbleCollection: () => SortAbleCollection,
  move: () => move
});
module.exports = __toCommonJS(move_exports);
var import_sequelize = require("sequelize");
var import_database = require("@nocobase/database");
var import_utils = require("../utils");
async function move(ctx, next) {
  const repository = ctx.databaseRepository || (0, import_utils.getRepositoryFromParams)(ctx);
  const { sourceId, targetId, sortField, targetScope, sticky, method } = ctx.action.params;
  if (repository instanceof import_database.BelongsToManyRepository) {
    throw new Error("Sorting association as 'belongs-to-many' type is not supported.");
  }
  if (repository instanceof import_database.HasManyRepository) {
    const hasManyField = repository.sourceCollection.getField(repository.associationName);
    if (!hasManyField.options.sortable) {
      throw new Error(
        `association ${hasManyField.options.name} in ${repository.sourceCollection.name} is not sortable`
      );
    }
  }
  const sortAbleCollection = new SortAbleCollection(
    repository instanceof import_database.Repository ? repository.collection : repository.targetCollection,
    repository instanceof import_database.Repository ? sortField : `${repository.association.foreignKey}Sort`
  );
  if (sourceId && targetId) {
    await sortAbleCollection.move(sourceId, targetId, {
      insertAfter: method === "insertAfter"
    });
  }
  if (sourceId && targetScope) {
    await sortAbleCollection.changeScope(sourceId, targetScope, method);
  }
  if (sourceId && sticky) {
    await sortAbleCollection.sticky(sourceId);
  }
  ctx.body = "ok";
  await next();
}
__name(move, "move");
const _SortAbleCollection = class _SortAbleCollection {
  collection;
  field;
  scopeKey;
  constructor(collection, fieldName = "sort") {
    this.collection = collection;
    this.field = collection.getField(fieldName);
    if (!(this.field instanceof import_database.SortField)) {
      throw new Error(`${fieldName} is not a sort field`);
    }
    this.scopeKey = this.field.get("scopeKey");
  }
  // insert source position to target position
  async move(sourceInstanceId, targetInstanceId, options = {}) {
    const sourceInstance = await this.collection.repository.findById(sourceInstanceId);
    const targetInstance = await this.collection.repository.findById(targetInstanceId);
    if (this.scopeKey && sourceInstance.get(this.scopeKey) !== targetInstance.get(this.scopeKey)) {
      await sourceInstance.update({
        [this.scopeKey]: targetInstance.get(this.scopeKey)
      });
    }
    await this.sameScopeMove(sourceInstance, targetInstance, options);
  }
  async changeScope(sourceInstanceId, targetScope, method) {
    const sourceInstance = await this.collection.repository.findById(sourceInstanceId);
    const targetScopeValue = targetScope[this.scopeKey];
    if (targetScopeValue && sourceInstance.get(this.scopeKey) !== targetScopeValue) {
      await sourceInstance.update(
        {
          [this.scopeKey]: targetScopeValue
        },
        {
          silent: false
        }
      );
      if (method === "prepend") {
        await this.sticky(sourceInstanceId);
      }
    }
  }
  async sticky(sourceInstanceId) {
    const sourceInstance = await this.collection.repository.findById(sourceInstanceId);
    await sourceInstance.update(
      {
        [this.field.get("name")]: 0
      },
      {
        silent: true
      }
    );
  }
  async sameScopeMove(sourceInstance, targetInstance, options) {
    const fieldName = this.field.get("name");
    const sourceSort = sourceInstance.get(fieldName);
    let targetSort = targetInstance.get(fieldName);
    if (options.insertAfter) {
      targetSort = targetSort + 1;
    }
    const scopeValue = this.scopeKey ? sourceInstance.get(this.scopeKey) : null;
    let updateCondition;
    let change;
    if (targetSort > sourceSort) {
      updateCondition = {
        [import_sequelize.Op.gt]: sourceSort,
        [import_sequelize.Op.lte]: targetSort
      };
      change = -1;
    } else {
      updateCondition = {
        [import_sequelize.Op.lt]: sourceSort,
        [import_sequelize.Op.gte]: targetSort
      };
      change = 1;
    }
    const where = {
      [fieldName]: updateCondition
    };
    if (scopeValue) {
      where[this.scopeKey] = {
        [import_sequelize.Op.eq]: scopeValue
      };
    }
    await this.collection.model.increment(fieldName, {
      where,
      by: change,
      silent: true
    });
    await sourceInstance.update(
      {
        [fieldName]: targetSort
      },
      {
        silent: true
      }
    );
  }
};
__name(_SortAbleCollection, "SortAbleCollection");
let SortAbleCollection = _SortAbleCollection;
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  SortAbleCollection,
  move
});
